<?php

/*
 * ELMA User Profile
 *
 * author: Joerg Heinemann
 * website: http://enterprise-log-management-appliance.org
 * license: GPL v3
 *
 * /srv/www/htdocs/elma/addons/ELMA User Profile/pwdchange.php
 *
 */

defined('is_running') or die('Not an entry point...');

gpPlugin::incl('functions.inc','require_once');

class UserProfilePWDChange extends UserProfile {

	function UserProfilePWDChange(){

		global $languages, $install_language;

/* --------------------------------------------------------------------
-- Configurable script to change users in htpasswd
-----------------------------------------------------------------------
-- This script will change users in an htpasswd file using
-- the PEAR Basic Authentication library. It imports header/footer files
-- if they are found so you can customize the look.
--
-- Installation:
--   Copy this file into a directory that is web protected and
-- update the script to point to the path of your actual htpasswd file.
-- You will need to make the htpasswd file is writable by the web server
-- user.
--
-- Author: Dan Afonso
-- Based on: David A. Horner (DAH) http://dave.thehorners.com
-- License: Public Domain
---------------------------------------------------------------------- */

// Edit the following to setup your script
$HTPASSWD_PATH = "/etc/apache2/conf.d/htpasswd";
$REDIRECT_URL = "/user/";
$MIN_PASSLEN = 5;
$MAX_PASSLEN = -1;

if(!include('File/Passwd/Authbasic.php')) {
?>
<div class='bg-danger'><h1>Installation Error</h1>
Your PEAR configuration is missing some required modules. You can install these
by running:
<pre>
[machine ~]# <b>pear install Auth</b>
[machine ~]# <b>pear install File_Passwd</b>
</pre>
</div>
<?php
   exit();
}

if(strlen($_SERVER["REMOTE_USER"])==0) {
?>
<div class='bg-danger'><h1>Installation Error</h1>
This script must be run from a password protected directory on your web site.
Please visit <a href="http://wiki.apache.org/httpd/PasswordBasicAuth"> the
Apache Wiki</a> for more information
</div>
<?php
  exit(0);
}

/// Check if we can write to the htpasswd file
if (!is_writable($HTPASSWD_PATH)){
?>
<div class='bg-danger'><h1>Installation Error</h1>
Your htpasswd file is not writable. You can change this by either:
<pre>
[machine ~]> <b>chmod a+w <?php echo $HTPASSWD_PATH; ?></b>
</pre>
... or if you prefer more security, as root you can do:
<pre>
[machine ~]# <b>chown <?php
        $a = posix_getpwuid(posix_getuid());
        echo $a["name"] . ' ' . $HTPASSWD_PATH; ?></b>
</pre>
</div>
<?php
   exit();
}

// Start processing
$user=$_SERVER["REMOTE_USER"];
$save_changes=false;

$passfile = new File_Passwd_Authbasic($HTPASSWD_PATH);
$ret = $passfile->load();
if(!PEAR::isError($ret)) {
  echo "<p>Hello $user,</p>";
  echo "<p>welcome to the htpasswd password change.</p>";
  echo "<p class='bg-warning'>For changing ldap or radius user passwords use your corporate password change utilities.</p>";
  echo "<h1>htpasswd Password Change</h1>";
  $verified=false;
  if(isset($_REQUEST["currpass"]) && strlen($_REQUEST["currpass"])) {
    // The PEAR library does not handle the (defaut) crypt() password
    // hash. So we have to decode it manually. Luckally, all the SHA1
    // hashes start with {SHA1}, and '{' is not present in crypt()
    // hashes.
    $users = $passfile->listUser();
    $salt = substr($users[$_SERVER["REMOTE_USER"]], 0, 2);

    // Check SHA1 passwords
    if(isset($users[$_SERVER["REMOTE_USER"]]) &&
       $salt == '{S' &&
       $passfile->verifyPasswd($user,$_REQUEST["currpass"])
      ) {
      $verified=true;
      echo "Password validated via SHA1<br/>\n";
    }
    // Check crypt()
    elseif (isset($users[$_SERVER["REMOTE_USER"]]) &&
            (crypt($_REQUEST["currpass"], $salt) ==
             $users[$_SERVER["REMOTE_USER"]])) {
      $verified=true;
      echo "Password validated via crypt()<br/>\n";
    } else {
      echo "<div class='bg-danger'><b>Failed:</b> Your current password did not match </div><br>";
    }
   }

   // Make sure the 2 passwords match
   $confirmed=false;
   if(isset($_REQUEST["newpass"]) && strlen($_REQUEST["newpass"]) > $MIN_PASSLEN) {
      if(strcmp($_REQUEST["newpass"],$_REQUEST["newpass2"]) == 0) {
         $confirmed=true;
      } else {
         echo "<div class='bg-danger'><b>Failed</b>: new password and confirmed password don 't match.";
      }
   } else if($verified) {
      echo "<div class='bg-danger'><b>Failed</b>: you must supply a new password
             and it must be at least $MIN_PASSLEN characters long.</div>";
   }

   if($verified && $confirmed) {
      $passfile->changePasswd($user, $_REQUEST["newpass"]);
      $save_changes=true;
   } else {
?>
<form method="post" class="form-horizontal" role="form">
  <div class="form-group">
    <label for="currpass" class="col-sm-2 control-label">Current password:</label>
    <div class="col-xs-4">
      <input type="password" class="form-control" id="currpass" name="currpass" placeholder="Current password">
    </div>
  </div>
  <div class="form-group">
    <label for="newpass" class="col-sm-2 control-label">New password:</label>
    <div class="col-xs-4">
      <input type="password" class="form-control" id="newpass" name="newpass" placeholder="New password">
    </div>
  </div>
  <div class="form-group">
    <label for="newpass2" class="col-sm-2 control-label">Re-type new password:</label>
    <div class="col-xs-4">
      <input type="password" class="form-control" id="newpass2" name="newpass2" placeholder="Re-type new password">
    </div>
  </div>
  <div class="form-group">
    <div class="col-sm-offset-2 col-sm-10">
      <button type="submit" class="btn btn-default">Change Password</button>
    </div>
  </div>
</form>
<?php
   }
   if($save_changes) {
      echo "Saving password file...<br/>";
      $ret=$passfile->save();
      if(!PEAR::isError($ret)) {
         echo "Password file successfully updated!<br/>";
      } else {
         echo "<div class='bg-danger'><b>Failed</b>: Failed to write password file!<br>";
         echo "(".$ret->getMessage().")<br></div>";
      }
   }

} else {
   echo "<div class='bg-danger'><b>Failed</b>:Failed to open password file!<br>";
   echo "(".$ret->getMessage().")<br></div>";
}

	} 
}

?>

